@@ -7,26 +7,20 @@ The xattr type wraps a path or file descriptor with a dict-like interface
 that exposes these extended attributes.
 """
 
-__version__ = '0.6.1'
-from constants import XATTR_NOFOLLOW, XATTR_CREATE, XATTR_REPLACE, \
-    XATTR_NOSECURITY, XATTR_MAXNAMELEN, XATTR_FINDERINFO_NAME, \
-    XATTR_RESOURCEFORK_NAME
+__version__ = '0.7.0'
 
-import _xattr
+from .lib import (XATTR_NOFOLLOW, XATTR_CREATE, XATTR_REPLACE,
+    XATTR_NOSECURITY, XATTR_MAXNAMELEN, XATTR_FINDERINFO_NAME,
+    XATTR_RESOURCEFORK_NAME, _getxattr, _fgetxattr, _setxattr, _fsetxattr,
+    _removexattr, _fremovexattr, _listxattr, _flistxattr)
 
-def _pyflakes_api():
-    # trick pyflakes into thinking these are used.
-    return [
-        XATTR_NOFOLLOW, XATTR_CREATE, XATTR_REPLACE,
-        XATTR_NOSECURITY, XATTR_MAXNAMELEN, XATTR_FINDERINFO_NAME,
-        XATTR_RESOURCEFORK_NAME,
-    ]
- 
 
-def _boundfunc(func, first):
-    def _func(*args):
-        return func(first, *args)
-    return _func
+__all__ = [
+    "XATTR_NOFOLLOW", "XATTR_CREATE", "XATTR_REPLACE", "XATTR_NOSECURITY",
+    "XATTR_MAXNAMELEN", "XATTR_FINDERINFO_NAME", "XATTR_RESOURCEFORK_NAME",
+    "xattr", "listxattr", "getxattr", "setxattr", "removexattr"
+]
+
 
 class xattr(object):
     """
@@ -45,73 +39,64 @@ class xattr(object):
         """
         self.obj = obj
         self.options = options
-        self.flavor = None
         fileno = getattr(obj, 'fileno', None)
         if fileno is not None:
-            obj = fileno()
-        if isinstance(obj, int):
-            self.flavor = 'fd'
-            self._bind_any('f%sxattr', obj, options)
+            self.value = fileno()
         else:
-            self.flavor = 'file'
-            self._bind_any('%sxattr', obj, options)
+            self.value = obj
 
     def __repr__(self):
-        if self.flavor:
-            return '<%s %s=%r>' % (type(self).__name__, self.flavor, self.obj)
+        if isinstance(self.value, int):
+            flavor = "fd"
         else:
-            return object.__repr__(self)
-    
-    def _bind_any(self, fmt, obj, options):
-        options = self.options
-        for method in ("get", "set", "remove", "list"):
-            name = '_' + method
-            func = getattr(_xattr, fmt % (method,))
-            meth = _boundfunc(func, obj)
-            try:
-                meth.__name__ = name
-            except TypeError:
-                pass
-            setattr(self, name, meth)
+            flavor = "file"
+        return "<%s %s=%r>" % (type(self).__name__, flavor, self.value)
+
+    def _call(self, name_func, fd_func, *args):
+        if isinstance(self.value, int):
+            return fd_func(self.value, *args)
+        else:
+            return name_func(self.value, *args)
 
     def get(self, name, options=0):
         """
         Retrieve the extended attribute ``name`` as a ``str``.
         Raises ``IOError`` on failure.
-        
+
         See x-man-page://2/getxattr for options and possible errors.
         """
-        return self._get(name, 0, 0, options | self.options)
+        return self._call(_getxattr, _fgetxattr, name, 0, 0, options | self.options)
 
     def set(self, name, value, options=0):
         """
         Set the extended attribute ``name`` to ``value``
         Raises ``IOError`` on failure.
-        
+
         See x-man-page://2/setxattr for options and possible errors.
         """
-        self._set(name, value, 0, options | self.options)
+        return self._call(_setxattr, _fsetxattr, name, value, 0, options | self.options)
 
     def remove(self, name, options=0):
         """
         Remove the extended attribute ``name``
         Raises ``IOError`` on failure.
-        
+
         See x-man-page://2/removexattr for options and possible errors.
         """
+        return self._call(_removexattr, _fremovexattr, name, options | self.options)
         self._remove(name, options | self.options)
 
     def list(self, options=0):
         """
         Retrieves the extended attributes currently set as a list
         of unicode strings.  Raises ``IOError`` on failure.
-        
+
         See x-man-page://2/listxattr for options and possible errors.
         """
-        res = self._list(options | self.options).split('\x00')
+        res = self._call(_listxattr, _flistxattr, options | self.options).split('\x00')
         res.pop()
         return [unicode(s, 'utf-8') for s in res]
-   
+
     # dict-like methods
 
     def __len__(self):
@@ -122,7 +107,7 @@ class xattr(object):
             self.remove(item)
         except IOError:
             raise KeyError(item)
-    
+
     def __setitem__(self, item, value):
         self.set(item, value)
 
@@ -131,7 +116,7 @@ class xattr(object):
             return self.get(item)
         except IOError:
             raise KeyError(item)
-        
+
     def iterkeys(self):
         return iter(self.list())
 
@@ -156,7 +141,7 @@ class xattr(object):
             seq = dict(seq)
         for k, v in seq.iteritems():
             self[k] = v
-    
+
     def copy(self):
         return dict(self.iteritems())
 
@@ -166,7 +151,7 @@ class xattr(object):
         except IOError:
             self[k] = d
         return d
-    
+
     def keys(self):
         return self.list()
 
@@ -186,20 +171,19 @@ class xattr(object):
 
 
 def listxattr(f, symlink=False):
-    __doc__ = xattr.list.__doc__
     return tuple(xattr(f).list(options=symlink and XATTR_NOFOLLOW or 0))
 
+
 def getxattr(f, attr, symlink=False):
-    __doc__ = xattr.get.__doc__
     return xattr(f).get(attr, options=symlink and XATTR_NOFOLLOW or 0)
 
+
 def setxattr(f, attr, value, options=0, symlink=False):
-    __doc__ = xattr.set.__doc__
     if symlink:
         options |= XATTR_NOFOLLOW
     return xattr(f).set(attr, value, options=options)
 
+
 def removexattr(f, attr, symlink=False):
-    __doc__ = xattr.remove.__doc__
     options = symlink and XATTR_NOFOLLOW or 0
     return xattr(f).remove(attr, options=options)
